"use strict";

var _require = require('path'),
    join = _require.join;

var _require2 = require('loader-utils'),
    getOptions = _require2.getOptions;

var historyMemory = {
  hash: 'createHashHistory',
  memory: 'createMemoryHistory',
  browser: 'createBrowserHistory'
};
/**
 * Universal App Shell Loader for Rax.
 */

module.exports = function (content) {
  var _this = this;

  var options = getOptions(this) || {};
  var renderMoudle = options.renderModule || 'rax';

  if (this.data.appConfig !== null) {
    var _this$data$appConfig = this.data.appConfig,
        routes = _this$data$appConfig.routes,
        _this$data$appConfig$ = _this$data$appConfig.historyType,
        historyType = _this$data$appConfig$ === void 0 ? 'hash' : _this$data$appConfig$;

    if (!Array.isArray(routes)) {
      this.emitError(new Error('Unsupported field in app.json: routes.'));
      routes = [];
    }

    var fixRootStyle = '';
    /**
     * Weex only support memory history.
     */

    if (options.type === 'weex') historyType = 'memory';
    /**
     * Web only compatible with 750rpx.
     */

    if (options.type === 'web') {
      var mutiple = options.mutiple || 100;
      fixRootStyle = "\n        const html = document.documentElement;\n        html.style.fontSize = html.clientWidth / 750 * " + mutiple + " + 'px';\n      ";
    }
    /**
     * Example format of routes:
     * [
     *  {
     *    "path": "/page1",
     *    "component": "pages/page1"
     *  }
     * ]
     */


    var assembleRoutes = routes.map(function (route, index) {
      return "routes.push({\n        index: " + index + ",\n        path: '" + route.path + "',\n        component: interopRequire(require('" + getDepPath(route.component, _this.rootContext) + "')),\n      });";
    }).join('\n');
    var source = "\n      import definedApp from '" + this.resourcePath + "';\n      import { render, createElement } from '" + renderMoudle + "';\n      import { " + historyMemory[historyType] + " as createHistory } from 'history';\n      import { _invokeAppCycle } from 'universal-app-runtime';\n      import DriverUniversal from 'driver-universal';\n      \n      const interopRequire = (mod) => mod && mod.__esModule ? mod.default : mod;\n\n      const getRouterConfig = () => {\n        const routes = [];\n        " + assembleRoutes + "\n        return {\n          history: createHistory(),\n          routes,\n        };\n      };\n\n      " + "\n      const appProps = {\n        routerConfig: getRouterConfig(),\n      };\n      \n      " + fixRootStyle + "\n      \n      let appLaunched = false;\n      \n      function Entry() {\n        const app = definedApp(appProps);\n        const startOptions = {};\n\n        if (!appLaunched) {\n          appLaunched = true;\n          _invokeAppCycle('launch', startOptions);\n        }\n\n        return app;\n      }\n      \n      render(createElement(Entry), null, { driver: DriverUniversal });\n    ";
    return source;
  } else {
    return content;
  }
};
/**
 * Get app.json content at picth loader.
 * @param remainingRequest
 * @param precedingRequest
 * @param data
 */


module.exports.pitch = function (remainingRequest, precedingRequest, data) {
  data.appConfig = null; // Provide default value.

  try {
    var configPath = this.resourcePath.replace(/\.js$/, '.json');
    var rawContent = this.fs.readFileSync(configPath).toString();
    data.appConfig = JSON.parse(rawContent);
    this.addDependency(configPath);
  } catch (err) {
    throw new Error('Can not get app.json, please check.');
  }
};
/**
 * ./pages/foo -> based on src, return original
 * /pages/foo -> based on rootContext
 * pages/foo -> based on src, add prefix: './'
 */


function getDepPath(path, rootContext) {
  if (rootContext === void 0) {
    rootContext = '';
  }

  if (path[0] === '.') {
    return path;
  } else if (path[0] === '/') {
    return join(rootContext, path);
  } else {
    return './' + path;
  }
}